"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }// src/index.ts
var _pluginsyntaxjsx = require('@babel/plugin-syntax-jsx'); var _pluginsyntaxjsx2 = _interopRequireDefault(_pluginsyntaxjsx);
var _parser = require('@babel/parser');

// src/transform.ts
var _compiler = require('@vue-jsx-vapor/compiler');

var _sourcemapjs = require('source-map-js');
var _traverse2 = require('@babel/traverse'); var _traverse3 = _interopRequireDefault(_traverse2);

// src/utils.ts
function isConditionalExpression(path) {
  return !!(path && (_optionalChain([path, 'optionalAccess', _ => _.type]) === "LogicalExpression" || path.type === "ConditionalExpression") && (path.parent.type === "JSXExpressionContainer" || path.parent.type === "ConditionalExpression" && isConditionalExpression(path.parentPath)));
}
function isJSXElement(node) {
  return !!node && (node.type === "JSXElement" || node.type === "JSXFragment");
}

// src/transform.ts
var traverse = _traverse3.default.default || _traverse3.default;
var transformJSX = (path, state) => {
  if (isJSXElement(path.parent)) return;
  const root = state.roots.shift();
  if (!root || !root.inVaporComponent) return;
  const isTS = _optionalChain([state, 'access', _2 => _2.filename, 'optionalAccess', _3 => _3.endsWith, 'call', _4 => _4("tsx")]);
  let { code, helpers, preamble, map } = _compiler.compile.call(void 0, root.node, {
    isTS,
    filename: state.filename,
    sourceMap: true,
    source: " ".repeat(root.node.start || 0) + root.source,
    ...state.opts.compile
  });
  helpers.forEach((helper) => state.importSet.add(helper));
  preamble = preamble.replaceAll(
    /(?<=const )t(?=(\d))/g,
    `_t${state.preambleIndex}`
  );
  code = code.replaceAll(/(?<== )t(?=\d)/g, `_t${state.preambleIndex}`);
  state.preambleIndex++;
  for (const [, key, value] of preamble.matchAll(
    /const (_t\d+) = (_template\(.*\))/g
  )) {
    const result = state.preambleMap.get(value);
    if (result) {
      code = code.replaceAll(key, result);
    } else {
      state.preambleMap.set(value, key);
    }
  }
  for (const [, events] of preamble.matchAll(/_delegateEvents\((.*)\)/g)) {
    events.split(", ").forEach((event) => state.delegateEventSet.add(event));
  }
  const ast = _parser.parse.call(void 0, `(() => {${code}})()`, {
    sourceFilename: state.filename,
    plugins: isTS ? ["jsx", "typescript"] : ["jsx"]
  });
  if (map) {
    const consumer = new (0, _sourcemapjs.SourceMapConsumer)(map);
    traverse(ast, {
      Identifier({ node: id }) {
        if (!id.loc) return;
        const originalLoc = consumer.originalPositionFor(id.loc.start);
        if (originalLoc.column) {
          id.loc.start.line = originalLoc.line;
          id.loc.start.column = originalLoc.column;
          id.loc.end.line = originalLoc.line;
          id.loc.end.column = originalLoc.column + id.name.length;
        }
      }
    });
  }
  path.replaceWith(ast.program.body[0]);
};

// src/index.ts
var SyntaxJSX = _pluginsyntaxjsx2.default.default || _pluginsyntaxjsx2.default;
var src_default = () => {
  return {
    name: "Vue JSX Vapor",
    inherits: SyntaxJSX,
    visitor: {
      JSXElement: transformJSX,
      JSXFragment: transformJSX,
      Program: {
        enter: (path, state) => {
          state.importSet = /* @__PURE__ */ new Set();
          state.delegateEventSet = /* @__PURE__ */ new Set();
          state.preambleMap = /* @__PURE__ */ new Map();
          state.preambleIndex = 0;
          state.roots = [];
          const collectRoot = (path2) => {
            if (!isJSXElement(path2.parent) && !isConditionalExpression(path2.parentPath)) {
              state.roots.push({
                node: path2.node,
                source: path2.getSource(),
                inVaporComponent: !state.opts.interop ? true : _optionalChain([path2, 'access', _5 => _5.findParent, 'call', _6 => _6(
                  ({ node }) => node.type === "CallExpression" && node.callee.type === "Identifier" && ["defineVaporComponent", "defineComponent"].includes(
                    node.callee.name
                  )
                ), 'optionalAccess', _7 => _7.node, 'access', _8 => _8.callee, 'access', _9 => _9.name]) === "defineVaporComponent"
              });
            }
          };
          path.traverse({
            JSXElement: collectRoot,
            JSXFragment: collectRoot
          });
        },
        exit: (path, state) => {
          const { delegateEventSet, importSet, preambleMap } = state;
          const statements = [];
          if (delegateEventSet.size) {
            statements.unshift(
              `_delegateEvents(${Array.from(delegateEventSet).join(", ")});`
            );
          }
          if (preambleMap.size) {
            let preambleResult = "";
            for (const [value, key] of preambleMap) {
              preambleResult += `const ${key} = ${value}
`;
            }
            statements.unshift(preambleResult);
          }
          const helpers = ["setText", "createTextNode"].filter((helper) => {
            const result = importSet.has(helper);
            result && importSet.delete(helper);
            return result;
          });
          if (helpers.length) {
            statements.unshift(
              `import { ${helpers.map((i) => `${i} as _${i}`).join(", ")} } from 'unplugin-vue-jsx-vapor/helper.js';
`
            );
          }
          if (importSet.size) {
            const importResult = Array.from(importSet).map((i) => `${i} as _${i}`).join(", ");
            statements.unshift(`import { ${importResult} } from 'vue';
`);
          }
          path.node.body.unshift(
            ..._parser.parse.call(void 0, statements.join("\n"), { sourceType: "module" }).program.body
          );
        }
      }
    }
  };
};


exports.default = src_default;

module.exports = exports.default;
