// src/index.ts
import _SyntaxJSX from "@babel/plugin-syntax-jsx";
import { parse as parse2 } from "@babel/parser";

// src/transform.ts
import { compile } from "@vue-jsx-vapor/compiler";
import { parse } from "@babel/parser";
import { SourceMapConsumer } from "source-map-js";
import _traverse from "@babel/traverse";

// src/utils.ts
function isConditionalExpression(path) {
  return !!(path && (path?.type === "LogicalExpression" || path.type === "ConditionalExpression") && (path.parent.type === "JSXExpressionContainer" || path.parent.type === "ConditionalExpression" && isConditionalExpression(path.parentPath)));
}
function isJSXElement(node) {
  return !!node && (node.type === "JSXElement" || node.type === "JSXFragment");
}

// src/transform.ts
var traverse = _traverse.default || _traverse;
var transformJSX = (path, state) => {
  if (isJSXElement(path.parent)) return;
  const root = state.roots.shift();
  if (!root || !root.inVaporComponent) return;
  const isTS = state.filename?.endsWith("tsx");
  let { code, helpers, preamble, map } = compile(root.node, {
    isTS,
    filename: state.filename,
    sourceMap: true,
    source: " ".repeat(root.node.start || 0) + root.source,
    ...state.opts.compile
  });
  helpers.forEach((helper) => state.importSet.add(helper));
  preamble = preamble.replaceAll(
    /(?<=const )t(?=(\d))/g,
    `_t${state.preambleIndex}`
  );
  code = code.replaceAll(/(?<== )t(?=\d)/g, `_t${state.preambleIndex}`);
  state.preambleIndex++;
  for (const [, key, value] of preamble.matchAll(
    /const (_t\d+) = (_template\(.*\))/g
  )) {
    const result = state.preambleMap.get(value);
    if (result) {
      code = code.replaceAll(key, result);
    } else {
      state.preambleMap.set(value, key);
    }
  }
  for (const [, events] of preamble.matchAll(/_delegateEvents\((.*)\)/g)) {
    events.split(", ").forEach((event) => state.delegateEventSet.add(event));
  }
  const ast = parse(`(() => {${code}})()`, {
    sourceFilename: state.filename,
    plugins: isTS ? ["jsx", "typescript"] : ["jsx"]
  });
  if (map) {
    const consumer = new SourceMapConsumer(map);
    traverse(ast, {
      Identifier({ node: id }) {
        if (!id.loc) return;
        const originalLoc = consumer.originalPositionFor(id.loc.start);
        if (originalLoc.column) {
          id.loc.start.line = originalLoc.line;
          id.loc.start.column = originalLoc.column;
          id.loc.end.line = originalLoc.line;
          id.loc.end.column = originalLoc.column + id.name.length;
        }
      }
    });
  }
  path.replaceWith(ast.program.body[0]);
};

// src/index.ts
var SyntaxJSX = _SyntaxJSX.default || _SyntaxJSX;
var src_default = () => {
  return {
    name: "Vue JSX Vapor",
    inherits: SyntaxJSX,
    visitor: {
      JSXElement: transformJSX,
      JSXFragment: transformJSX,
      Program: {
        enter: (path, state) => {
          state.importSet = /* @__PURE__ */ new Set();
          state.delegateEventSet = /* @__PURE__ */ new Set();
          state.preambleMap = /* @__PURE__ */ new Map();
          state.preambleIndex = 0;
          state.roots = [];
          const collectRoot = (path2) => {
            if (!isJSXElement(path2.parent) && !isConditionalExpression(path2.parentPath)) {
              state.roots.push({
                node: path2.node,
                source: path2.getSource(),
                inVaporComponent: !state.opts.interop ? true : path2.findParent(
                  ({ node }) => node.type === "CallExpression" && node.callee.type === "Identifier" && ["defineVaporComponent", "defineComponent"].includes(
                    node.callee.name
                  )
                )?.node.callee.name === "defineVaporComponent"
              });
            }
          };
          path.traverse({
            JSXElement: collectRoot,
            JSXFragment: collectRoot
          });
        },
        exit: (path, state) => {
          const { delegateEventSet, importSet, preambleMap } = state;
          const statements = [];
          if (delegateEventSet.size) {
            statements.unshift(
              `_delegateEvents(${Array.from(delegateEventSet).join(", ")});`
            );
          }
          if (preambleMap.size) {
            let preambleResult = "";
            for (const [value, key] of preambleMap) {
              preambleResult += `const ${key} = ${value}
`;
            }
            statements.unshift(preambleResult);
          }
          const helpers = ["setText", "createTextNode"].filter((helper) => {
            const result = importSet.has(helper);
            result && importSet.delete(helper);
            return result;
          });
          if (helpers.length) {
            statements.unshift(
              `import { ${helpers.map((i) => `${i} as _${i}`).join(", ")} } from 'unplugin-vue-jsx-vapor/helper.js';
`
            );
          }
          if (importSet.size) {
            const importResult = Array.from(importSet).map((i) => `${i} as _${i}`).join(", ");
            statements.unshift(`import { ${importResult} } from 'vue';
`);
          }
          path.node.body.unshift(
            ...parse2(statements.join("\n"), { sourceType: "module" }).program.body
          );
        }
      }
    }
  };
};
export {
  src_default as default
};
